from __future__ import print_function

from contextlib import contextmanager
import logging
import os
import subprocess
from argparse import Namespace

from conda.api import get_index
from conda_build.metadata import MetaData
import conda.config
import conda_build.config


#import conda_build_all.logging
import conda_build_all.version_matrix as vn_matrix


@contextmanager
def setup_vn_mtx_case(case):
    orig_npy = conda_build.config.config.CONDA_NPY
    orig_py = conda_build.config.config.CONDA_PY

    for pkg, version in case:
        version = int(version.replace('.', ''))
        if pkg == 'python':
            conda_build.config.config.CONDA_PY = version
        elif pkg == 'numpy':
            conda_build.config.config.CONDA_NPY = version
        else:
            raise NotImplementedError('Package {} not yet implemented.'
                                      ''.format(pkg))
    yield
    conda_build.config.config.CONDA_NPY = orig_npy
    conda_build.config.config.CONDA_PY = orig_py


class ResolvedDistribution(object):
    """
    Represents a conda pacakge, with the appropriate special case
    versions fixed (e.g. CONDA_PY, CONDA_NPY). Without this, a meta
    changes as the conda_build.config.CONDA_NPY changes.

    Parameters
    ----------
    meta: conda_build.metadata.MetData
        The package which has been resolved.
    special_versions: iterable
        A list of the versions which have been resolved for this package.
        e.g. ``(['python', '27'],)``

    """
    def __init__(self, meta, special_versions=()):
        self.meta = meta
        self.special_versions = special_versions

    def __repr__(self):
        return 'BakedDistribution({}, {})'.format(self.meta,
                                                  self.special_versions)

    def __str__(self):
        return self.dist()

    def vn_context(self):
        return setup_vn_mtx_case(self.special_versions)

    def __getattr__(self, name):
        with setup_vn_mtx_case(self.special_versions):
            self.meta.parse_again()
            result = getattr(self.meta, name)

        # Wrap any callable such that it is called within the appropriate
        # environment.
        # callable exists in python 2.* and >=3.2
        if callable(result):
            orig_result = result
            import functools
            @functools.wraps(result)
            def with_vn_mtx_setup(*args, **kwargs):
                with setup_vn_mtx_case(self.special_versions):
                    self.meta.parse_again()
                    return orig_result(*args, **kwargs)
            result = with_vn_mtx_setup
        return result

    @classmethod
    def resolve_all(cls, meta, index=None, extra_conditions=None):
        """
        Given a package, return a list of ResolvedDistributions, one for each
        possible (necessary) version permutation.

        """
        if index is None:
            with vn_matrix.override_conda_logging('WARN'):
                index = get_index()

        cases = sorted(vn_matrix.special_case_version_matrix(meta, index))

        if extra_conditions:
            cases = list(vn_matrix.filter_cases(cases, extra_conditions))
        result = []
        for case in cases:
            dist = cls(meta, case)
            if not dist.skip():
                result.append(dist)
        return result





from argparse import Namespace
from contextlib import contextmanager
import logging
import mock
import os
import shutil
import sys
import tempfile
import unittest


from conda_build_all.tests.unit.dummy_index import DummyIndex, DummyPackage
from conda_build_all.artefact_destination import (ArtefactDestination,
                                                  AnacondaClientChannelDest,
                                                  DirectoryDestination)
import conda_build_all.artefact_destination


class Test_AnacondaClientChannelDest(unittest.TestCase):
    # These tests make extensive use of mock to avoid the need to contact the
    # conda.anaconda.org server.
    # Integration tests which do use the server are available for inspect_binstar.
    def setUp(self):
        self.logger_patch = mock.patch('conda_build_all.artefact_destination.log')
        self.logger = self.logger_patch.start()

    def tearDown(self):
        self.logger_patch.stop()

    @contextmanager
    def dist_exists_setup(self, on_owner, on_channel):
        dist_exists = mock.patch('conda_build_all.inspect_binstar.distribution_exists', return_value=on_owner)
        dist_exists_on_channel = mock.patch('conda_build_all.inspect_binstar.distribution_exists_on_channel', return_value=on_channel)
        with dist_exists:
            with dist_exists_on_channel:
                yield

    def test_not_already_available_not_just_built(self):
        client, owner, channel = [mock.sentinel.client, mock.sentinel.owner,
                                  mock.sentinel.channel]
        ad = AnacondaClientChannelDest(mock.sentinel.token, owner, channel)
        ad._cli = client
        meta = DummyPackage('a', '2.1.0')
        with self.dist_exists_setup(on_owner=True, on_channel=False):
            with mock.patch('conda_build_all.inspect_binstar.add_distribution_to_channel') as add_to_channel:
                ad.make_available(meta, mock.sentinel.dist_path,
                                  just_built=False)
        add_to_channel.assert_called_once_with(client, owner, meta, channel=channel)
        self.logger.info.assert_called_once_with('Adding existing a-0.0-0 to the sentinel.owner/sentinel.channel channel.')

    def test_not_already_available_just_built(self):
        client, owner, channel = [mock.sentinel.client, mock.sentinel.owner,
                                  mock.sentinel.channel]
        ad = AnacondaClientChannelDest(mock.sentinel.token, owner, channel)
        ad._cli = client
        meta = DummyPackage('a', '2.1.0')
        with self.dist_exists_setup(on_owner=False, on_channel=False):
            with mock.patch('conda_build_all.build.upload') as upload:
                ad.make_available(meta, mock.sentinel.dist_path,
                                  just_built=True)
        upload.assert_called_once_with(client, meta, owner, channels=[channel])
        self.logger.info.assert_called_once_with('Uploading a to the sentinel.channel channel.')

    def test_already_available_not_just_built(self):
        # Note, we exercise the use of get_binstar here too.

        client, owner, channel = [mock.sentinel.client, mock.sentinel.owner,
                                  mock.sentinel.channel]
        ad = AnacondaClientChannelDest(mock.sentinel.token, owner, channel)
        meta = DummyPackage('a', '2.1.0')
        with self.dist_exists_setup(on_owner=True, on_channel=True):
            with mock.patch('binstar_client.utils.get_binstar') as get_binstar:
                ad.make_available(meta, mock.sentinel.dist_path, just_built=False)
        get_binstar.assert_called_once_with(Namespace(site=None, token=mock.sentinel.token))
        # Nothing happens, we just get a message.
        self.logger.info.assert_called_once_with('Nothing to be done for a - it is already on sentinel.owner/sentinel.channel.')

    def test_already_available_not_just_built(self):
        client, owner, channel = [mock.sentinel.client, mock.sentinel.owner,
                                  mock.sentinel.channel]
        ad = AnacondaClientChannelDest(mock.sentinel.token, owner, channel)
        ad._cli = client
        meta = DummyPackage('a', '2.1.0')
        with self.dist_exists_setup(on_owner=True, on_channel=True):
            ad.make_available(meta, mock.sentinel.dist_path, just_built=True)
        # Nothing happens, we just get a message.
        self.logger.warn.assert_called_once_with("Assuming the distribution we've just built and the one on sentinel.owner/sentinel.channel are the same.")

    def test_already_available_elsewhere(self):
        client, owner, channel = [mock.sentinel.client, mock.sentinel.owner,
                                  mock.sentinel.channel]
        ad = AnacondaClientChannelDest(mock.sentinel.token, owner, channel)
        ad._cli = client
        meta = DummyPackage('a', '2.1.0')
        with self.dist_exists_setup(on_owner=False, on_channel=False):
            with self.assertRaises(NotImplementedError):
                ad.make_available(meta, mock.sentinel.dist_path, just_built=False)

    def test_from_spec_owner(self):
        spec = 'testing'
        os.environ['BINSTAR_TOKEN'] = 'a test token'
        dest = AnacondaClientChannelDest.from_spec(spec)
        self.assertEqual(dest.token, 'a test token')
        self.assertEqual(dest.owner, 'testing')
        self.assertEqual(dest.channel, 'main')

    def test_from_spec_owner_and_channel(self):
        spec = 'testing_owner/channels/my_channel'
        os.environ['BINSTAR_TOKEN'] = 'a test token'
        dest = AnacondaClientChannelDest.from_spec(spec)
        self.assertEqual(dest.token, 'a test token')
        self.assertEqual(dest.owner, 'testing_owner')
        self.assertEqual(dest.channel, 'my_channel')


class Test_DirectoryDestination(unittest.TestCase):
    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp(prefix='recipes')

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_not_copying(self):
        dd = DirectoryDestination(self.tmp_dir)
        dd.make_available(mock.sentinel.dummy_meta, mock.sentinel.dummy_path,
                          just_built=False)

    def test_copying(self):
        dd = DirectoryDestination(self.tmp_dir)
        with mock.patch('shutil.copy') as copy:
            dd.make_available(mock.sentinel.dummy_meta, mock.sentinel.dummy_path,
                              just_built=True)
        copy.assert_called_once_with(mock.sentinel.dummy_path, self.tmp_dir)


if __name__ == '__main__':
    unittest.main()



# NOTE: This module has no unit tests.

import conda.config
import binstar_client
from conda_build.build import bldpkg_path


def distribution_exists(binstar_cli, owner, metadata):
    """
    Determine whether a distribution exists.

    This does not check specific channels - it is either on binstar or it is not.
    """
    fname = '{}/{}.tar.bz2'.format(conda.config.subdir, metadata.dist())
    try:
        r = binstar_cli.distribution(owner, metadata.name(), metadata.version(),
                                     fname)
        exists = True
    except binstar_client.NotFound:
        exists = False
    return exists


def distribution_exists_on_channel(binstar_cli, owner, metadata, channel='main'):
    """
    Determine whether a distribution exists on a specific channel.

    Note from @pelson: As far as I can see, there is no easy way to do this on binstar.

    """
    fname = '{}/{}.tar.bz2'.format(conda.config.subdir, metadata.dist())
    distributions_on_channel = [dist['basename'] for dist in
                                binstar_cli.show_channel(owner=owner, channel=channel)['files']]
    return fname in distributions_on_channel


def add_distribution_to_channel(binstar_cli, owner, metadata, channel='main'):
    """
    Add a(n already existing) distribution on binstar to another channel.
    
    Note - the addition is done based on name and version - no build strings etc.
    so if you have a foo-0.1-np18 and foo-0.1-np19 *both* will be added to the channel.
    
    """
    package_fname = '{}/{}.tar.bz2'.format(conda.config.subdir, metadata.dist())
    binstar_cli.add_channel(channel, owner, metadata.name(), metadata.version())#filename=package_fname)



import argparse
import logging
import os

import conda_build.config

import conda_build_all
import conda_build_all.builder
import conda_build_all.artefact_destination as artefact_dest


def main():
    parser = argparse.ArgumentParser(
                              description='Build many conda distributions.')

    parser.add_argument('--version', action='version',
                        version=conda_build_all.__version__,
                        help="Show conda-build-all's version, and exit.")

    parser.add_argument('recipes',
        help='The folder containing conda recipes to build.')
    parser.add_argument('--inspect-channels', nargs='*',
        help=('Skip a build if the equivalent disribution is already '
              'available in the specified channel.'))
    parser.add_argument('--inspect-directories', nargs='*',
        help=('Skip a build if the equivalent disribution is already '
              'available in the specified directory.'))
    parser.add_argument('--no-inspect-conda-bld-directory', default=False,
        action='store_true',
        help='Do not add the conda-build directory to the inspection list.')

    parser.add_argument('--artefact-directory',
        help='A directory for any newly built distributions to be placed.')
    parser.add_argument('--upload-channels', nargs='*', default=[],
        help=('The channel(s) to upload built distributions to (requires '
              'BINSTAR_TOKEN envioronment variable).'))

    parser.add_argument("--matrix-conditions", nargs='*', default=[],
        help=("Extra conditions for computing the build matrix "
              "(e.g. 'python 2.7.*'). Note, your build matrix may also be being "
              "limited by --matrix-max-n-major-versions and "
              "--matrix-max-n-minor-versions."))
    parser.add_argument("--matrix-max-n-major-versions", default=2, type=int,
        help=("When computing the build matrix, limit to the latest n major "
              "versions (0 makes this unlimited). For example, if Python 1, "
              "2 and Python 3 are resolved by the recipe and associated "
              "matrix conditions, only the latest N major version will be "
              "used for the build matrix. (default: 2) "))
    parser.add_argument("--matrix-max-n-minor-versions", default=2, type=int,
        help=("When computing the build matrix, limit to the latest n minor "
              "versions (0 makes this unlimited). Note that this does not "
              "limit the number of major versions (see also "
              "matrix-max-n-major-version). For example, if Python 2 and "
              "Python 3 are resolved by the recipe and associated matrix "
              "conditions, a total of Nx2 builds will be identified. "
              "(default: 2)"))

    args = parser.parse_args()

    matrix_conditions = args.matrix_conditions
    max_n_versions = (args.matrix_max_n_major_versions,
                      args.matrix_max_n_minor_versions)

    inspection_directories = args.inspect_directories or []
    if (not args.no_inspect_conda_bld_directory and
            os.path.isdir(conda_build.config.config.bldpkgs_dir)):
        inspection_directories.append(conda_build.config.config.bldpkgs_dir)

    artefact_destinations = []
    for channel in args.upload_channels:
        dest = artefact_dest.AnacondaClientChannelDest.from_spec(channel)
        artefact_destinations.append(dest)
    if args.artefact_directory:
        dest = artefact_dest.DirectoryDestination(args.artefact_directory)
        artefact_destinations.append(dest)

    artefact_dest.log.setLevel(logging.INFO)
    artefact_dest.log.addHandler(logging.StreamHandler())

    b = conda_build_all.builder.Builder(args.recipes, args.inspect_channels,
                                        inspection_directories,
                                        artefact_destinations,
                                        args.matrix_conditions,
                                        max_n_versions)
    b.main()


if __name__ == '__main__':
    main()



import os
import shutil
import tempfile
import unittest
import textwrap

import conda_build.config
from conda_build.metadata import MetaData

from conda_build_all.builder import list_metas
from conda_build_all.tests.integration.test_builder import RecipeCreatingUnit


class Test_list_metas(RecipeCreatingUnit):
    def setUp(self):
        super(Test_list_metas, self).setUp()
        m1 = self.write_meta('m1', """
            package:
                name: m1
            """)
        m2 = self.write_meta('.', """
            package:
                name: m2
            """)
        m3 = self.write_meta('d1/d2/d3/meta3', """
            package:
                name: m3
            """)
        m4 = self.write_meta('da1/da2/da3/meta4', """
            package:
                name: m4
            """)

    def test_depth_0(self):
        metas = list_metas(self.recipes_root_dir, max_depth=0)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m1', 'm2', 'm3', 'm4'])

    def test_depth_m1(self):
        metas = list_metas(self.recipes_root_dir, max_depth=-1)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m1', 'm2', 'm3', 'm4'])

    def test_depth_1(self):
        metas = list_metas(self.recipes_root_dir, max_depth=1)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m2'])

    def test_depth_2(self):
        metas = list_metas(self.recipes_root_dir, max_depth=2)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m1', 'm2'])

    def test_default_depth(self):
        metas = list_metas(self.recipes_root_dir)
        names = [meta.name() for meta in metas]
        self.assertEqual(sorted(names), ['m1', 'm2', 'm3', 'm4'])

   
if __name__ == '__main__':
    unittest.main()



import os
import unittest

from binstar_client.utils import get_binstar
from argparse import Namespace

from conda_build_all.build import build, upload
from conda_build_all.inspect_binstar import (distribution_exists,
                                             distribution_exists_on_channel,
                                             add_distribution_to_channel)
from conda_build_all.tests.integration.test_builder import RecipeCreatingUnit



def clear_binstar(cli, owner):
    """
    Empty all distributions for a user.

    The "rm -rf *" of the binstar world.

    """
    for channel in cli.list_channels(owner):
        cli.remove_channel(owner, channel)

    for package in cli.user_packages(owner):
        cli.remove_package(owner, package['name'])


OWNER = 'Obvious-ci-tests'
CLIENT = get_binstar(Namespace(token=os.environ.get('BINSTAR_TOKEN', None), site=None))


@unittest.skipIf(os.environ.get('CONDA_BUILD_ALL_TEST_ANACONDA_CLOUD', False) != '1',
                 "Not testing real binstar usage as the "
                 "CONDA_BUILD_ALL_TEST_ANACONDA_CLOUD environment variable is not "
                 "set to '1'.")
class Test(RecipeCreatingUnit):
    # Note: These tests upload things to anaconda.org and are completely global. That is,
    # if somebody else in the world is running the tests at the same time anywhere on the planet,
    # they will behave in very strange ways (highly likely to fail).
    def setUp(self):
        clear_binstar(CLIENT, OWNER)
        super(Test, self).setUp()

    def tearDown(self):
        clear_binstar(CLIENT, OWNER)
        super(Test, self).tearDown()

    def test_distribution_exists(self):
        # Build a recipe.
        meta = self.write_meta('test_recipe_1', """
                    package:
                        name: test_recipe_1
                        version: 'determined_at_build_time'
                    build:
                        script: echo "v0.1.0.dev1" > __conda_version__.txt
                    """)
        meta = build(meta)

        # Check distribution exists returns false when there is no distribution.
        self.assertFalse(distribution_exists(CLIENT, OWNER, meta))

        # upload the distribution 
        upload(CLIENT, meta, OWNER, channels=['testing'])

        # Check the distribution exists. Notice there is no channel being supplied here.
        self.assertTrue(distribution_exists(CLIENT, OWNER, meta))

        # Check the distribution is on testing but not on main.
        self.assertTrue(distribution_exists_on_channel(CLIENT, OWNER, meta, channel='testing'))
        self.assertFalse(distribution_exists_on_channel(CLIENT, OWNER, meta, channel='main'))

        add_distribution_to_channel(CLIENT, OWNER, meta, channel='main')
        # Check that the distribution has been added.
        self.assertTrue(distribution_exists_on_channel(CLIENT, OWNER, meta, channel='main'))


if __name__ == '__main__':
    unittest.main()



# There should be *very* few tests here. The intention of this module is to test the
# conda-build-all interface for a few cases to ensure that when pulling all of the
# components together, we get the desired behaviour. Please consider adding *unit*
# tests for new functionality.

from argparse import Namespace
from contextlib import contextmanager
import os
import shutil
import subprocess
import sys
import tempfile
import textwrap
import unittest

from binstar_client.utils import get_binstar
import conda.config

from conda_build_all.build import build, upload
from conda_build_all.inspect_binstar import (distribution_exists,
                                             distribution_exists_on_channel,
                                             add_distribution_to_channel)
from conda_build_all.tests.integration.test_builder import RecipeCreatingUnit
from conda_build_all.resolved_distribution import ResolvedDistribution


def clear_binstar(cli, owner):
    """
    Empty all distributions for a user.

    The "rm -rf *" of the binstar world.

    """
    for channel in cli.list_channels(owner):
        cli.remove_channel(owner, channel)

    for package in cli.user_packages(owner):
        cli.remove_package(owner, package['name'])


OWNER = 'Obvious-ci-tests'
CLIENT = get_binstar(Namespace(token=os.environ.get('BINSTAR_TOKEN', None), site=None))


@unittest.skipIf(os.environ.get('CONDA_BUILD_ALL_TEST_ANACONDA_CLOUD', False) != '1',
                 "Not testing real binstar usage as the "
                 "CONDA_BUILD_ALL_TEST_ANACONDA_CLOUD environment variable is not "
                 "set to '1'.")
class Test(RecipeCreatingUnit):
    # Note: These tests upload things to anaconda.org and are completely global. That is,
    # if somebody else in the world is running the tests at the same time anywhere on the planet,
    # they will behave in very strange ways (highly likely to fail).
    def setUp(self):
        clear_binstar(CLIENT, OWNER)
        self.conda_bld_root = tempfile.mkdtemp(prefix='conda_bld_dir_')
        super(Test, self).setUp()

    def tearDown(self):
        clear_binstar(CLIENT, OWNER)
        shutil.rmtree(self.conda_bld_root)
        super(Test, self).tearDown()

    @contextmanager
    def configure_conda(self):
        condarc = os.path.join(self.conda_bld_root, 'condarc')
        with open(condarc, 'w') as fh:
            fh.write(textwrap.dedent("""
                channels: []
                add_pip_as_python_dependency: False
                conda-build:
                    root-dir: {}
                """.format(self.conda_bld_root)))
        yield condarc

    def call(self, cmd_args):
        cmd = [sys.executable, '-m', 'conda_build_all.cli'] + list(cmd_args)
        environ = os.environ.copy()
        # Use a very limited condarc config.
        with self.configure_conda() as rc_path:
            environ['CONDARC'] = rc_path
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                                 env=environ, bufsize=0)
            line = p.stdout.readline()
            while line:
                print(str(line.rstrip().decode('utf8')))
                line = p.stdout.readline()
            # Wait for the return code.
            p.communicate()
        self.assertEqual(p.returncode, 0, 'Exit code was not 0 (got {})'.format(p.returncode))

    def test(self):
        # Build a recipe.
        py2 = self.write_meta('py1', """
                    package:
                        name: python
                        version: 1.2.3
                    """)
        py2 = self.write_meta('py2', """
                    package:
                        name: python
                        version: 2.1.10
                    """)
        a = self.write_meta('a', """
                    package:
                        name: a
                        version: 3.1.4
                    requirements:
                        build:
                            - python
                        run:
                            - python
                    """)

        a_py12 = ResolvedDistribution(a, (('python', '12', ), ))
        a_py21 = ResolvedDistribution(a, (('python', '21', ), ))
        a_py99 = ResolvedDistribution(a, (('python', '99', ), ))

        testing_channel = '{}/channel/{}'.format(OWNER, 'testing')
        self.call([self.recipes_root_dir, '--upload-channel', testing_channel])

        # Check that we have started on the right footing - the distribution should be on testing,
        # but not on main.
        self.assertTrue(distribution_exists_on_channel(CLIENT, OWNER, py2, channel='testing'))
        self.assertFalse(distribution_exists_on_channel(CLIENT, OWNER, py2, channel='main'))

        # Check that we've had a py21 and py12, but not a py99 for a.
        self.assertTrue(distribution_exists_on_channel(CLIENT, OWNER, a_py12, channel='testing'))
        self.assertTrue(distribution_exists_on_channel(CLIENT, OWNER, a_py21, channel='testing'))
        self.assertFalse(distribution_exists_on_channel(CLIENT, OWNER, a_py99, channel='testing'))

        # Remove the built distribution, re-run, and assert that we didn't bother re-building.
        dist_path = os.path.join(self.conda_bld_root, conda.config.subdir, a_py21.pkg_fn())
        self.assertTrue(os.path.exists(dist_path))
        os.remove(dist_path)
        self.call([self.recipes_root_dir, '--inspect-channel', testing_channel, '--upload-channel', testing_channel])
        self.assertFalse(os.path.exists(dist_path))

        # Now put a condition in. In this case, only build dists for py<2
        CLIENT.remove_dist(OWNER, a_py21.name(), a_py21.version(), '{}/{}'.format(conda.config.subdir, a_py21.pkg_fn()))
        self.assertFalse(distribution_exists_on_channel(CLIENT, OWNER, a_py21, channel='testing'))
        self.call([self.recipes_root_dir, '--inspect-channel', testing_channel, '--upload-channel', testing_channel,
                   '--matrix-condition', 'python <2'])
        self.assertFalse(distribution_exists_on_channel(CLIENT, OWNER, a_py21, channel='testing'))
        self.assertFalse(os.path.exists(dist_path))

        # Without the condition, we should be re-building the distribution
        self.call([self.recipes_root_dir, '--inspect-channel', testing_channel, '--upload-channel', testing_channel])
        self.assertTrue(os.path.exists(dist_path))
        self.assertTrue(distribution_exists_on_channel(CLIENT, OWNER, a_py21, channel='testing'))


if __name__ == '__main__':
    unittest.main()
